/* Cultural algorithm for constrained optimization */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <values.h>
#include <time.h>
#include <sys/timeb.h>
#include "randfunc.h"
/* Give the file name with the problem */
#include "probl01.h"

#define TRUSSPROBL 1

/* Give the following 2 parameters. Do not modify anything else in this file */
/* Population size */
#define TAMPOBL 20
/* How many individuals will take the acceptance function at each generation */
#define TOP 2

/* Clases de celdas de creencias */
#define SEMIFACTIBLE 1
#define FACTIBLE 2
#define DESCONOCIDA 3
#define NO_FACTIBLE 4

void violacionInic(float *);
void violacion(struct individuo *, float *);
void creenciasInic(struct creencias *);
void poblacionInic(struct individuo *, struct creencias *);
void actualizaCreencias(struct individuo *, struct creencias *, int);
void expande(struct celda *, struct individuo *, struct creencias *);
void aceptar(struct individuo *);
int compAptitud(const void *, const void *);
void generarHijos(struct individuo *, struct creencias *);
void mueve(int, int, float, struct individuo *, struct creencias *);
struct celda *cercana(int, struct celda *);
struct celda *busca(int, struct celda *);
void selecciona(struct individuo *);
int compVictorias(const void *, const void *);
void nuevoInd(struct individuo *);
void extremos(int *, int *, struct individuo *);

float media;
/* pacific standard & daylight savings */
char *tzstr = "TZ=PST8PDT";
struct timeb tiempo1, tiempo2;
int diferencia;

main(int argc, char **argv) {
	int i, j, indmin, indmax, Gmax;
	float semilla;
	float gmax[NUMCONSTR];
	char cadena[25];
	long evs = 0;
	FILE *a;
	struct creencias espcreencias;
	struct individuo pobl[2*TAMPOBL + 3];

	if (argc == 4) {
		Gmax = atoi(argv[1]);
		semilla = atof(argv[2]);
		strcpy(cadena, argv[3]);
	}
	else {
		printf("Teclee el nmero mximo de generaciones: ");
		scanf("%d", &Gmax);
		printf("Teclee la semilla de aleatorios (0..1): ");
		scanf("%f", &semilla);
		printf("Teclee el nombre del archivo de salida: ");
		scanf("%s", cadena);
	}


	if ( (a = fopen(cadena, "w")) == NULL ) {
		printf("Error al abrir el archivo de salida.\n");
		return (1);
	}
	fprintf(a, "Nmero mximo de generaciones: %d\n", Gmax);
	fprintf(a, "Semilla de aleatorios %f\n\n", semilla);

	putenv(tzstr);
	tzset();
	diferencia = 0;

	/* Inicializa aleatorios */
	randomizef(semilla);
	initrandomnormaldeviate();

	/* Inicia los valores gmax */
	violacionInic(gmax);

	/* Inicia el espacio de creencias */
	creenciasInic(&espcreencias);

	/* Crea una poblacin inicial aleatoria */
	poblacionInic(pobl, &espcreencias);

	/* Se evala la aptitud de cada individuo */
	for (i = 0; i < TAMPOBL; i++) {
		evalua(&(pobl[i]), 0, 0, 0);
		violacion(&(pobl[i]), gmax);
		evs++;
	}

	/* El ciclo principal, se ejecuta Gmax generaciones */
	for (j = 0; j < Gmax; j++) {

		/* Actualiza el espacio de creencias,
		   tanto normativas como de restricciones */
		actualizaCreencias(pobl, &espcreencias, j);

		/* Se generan los hijos mediante mutacin influenciada
		   por el espacio de creencias */
		generarHijos(pobl, &espcreencias);
		/* Se evala la aptitud de cada hijo */
		for (i = TAMPOBL; i < 2*TAMPOBL; i++) {
			evalua(&(pobl[i]), 0, 0, 0);
			violacion(&(pobl[i]), gmax);
			evs++;
		}

		/* Se hace el torneo entre padres e hijos */
		selecciona(pobl);
		
		/* Impresin de los datos de la generacin */
		if (j >= Gmax - 11 || (j+1)%(Gmax/100) == 0) {
		extremos(&indmin, &indmax, pobl);
		fprintf(a, "Generacin %d\n", j+1);
		fprintf(a, "Aptitud media: %0.7f, aptitud mxima: %0.7f, aptitud mnima: %0.7f\n", media, pobl[indmax].aptitud, pobl[indmin].aptitud);
		fprintf(a, "Mejor individuo:");
		for (i = 0; i < VARIABLES; i++) {
			fprintf(a, " %0.7f,", pobl[indmin].variable[i]);
		}
		for (i = 0; i < NUMCONSTR; i++) {
			fprintf(a, "\ng%d = %0.7f", i+1, pobl[indmin].g[i]);
		}
fprintf(a, "\nviol = %0.7f", pobl[indmin].viol);
		fprintf(a, "\nEl mejor individuo es ");
		if (!pobl[indmin].factible) {
			fprintf(a, "no ");
		}
		fprintf(a, "factible.\nEvluaciones: %ld\n\n", evs);
		}
	}
	fclose(a);
//	printf("Thousandths of a second: %d\n", diferencia);
	printf("%0.7f", pobl[indmin].aptitud);
	if (!pobl[indmin].factible) {
		printf(" i");
	}
	printf("\n");
	return (0);
}

/* Inicia el espacio de creencias, con los intervalos para cada variable. */
void creenciasInic(struct creencias *esp) {
	int i;
	float l[VARIABLES], u[VARIABLES];

	limites(l, u);
	for (i = 0; i < VARIABLES; i++) {
		esp->L[i] = esp->U[i] = MAXFLOAT;
		esp->l[i] = esp->lp[i] = l[i];
		esp->u[i] = esp->up[i] = u[i];
	}
	esp->raiz = (struct celda *) malloc(sizeof(struct celda));
	esp->raiz->padre = NULL;
	esp->raiz->hijo = NULL;
	esp->raiz->profundidad = PROFUNDIDAD_MAX;
}

void poblacionInic(struct individuo *pobl, struct creencias *esp) {
	int i, j;
	for (i = 0; i < TAMPOBL; i++) {
		/* Para cada variable se elige un nmero real,
		   en el rango especificado */
		for (j = 0; j < VARIABLES; j++) {
			pobl[i].variable[j] = rndreal(esp->lp[j], esp->up[j]);
		}
	}
}

void violacionInic(float gmax[]) {
	int i;

	for (i = 0; i < NUMCONSTR; i++) {
		gmax[i] = 0;
	}
}

void violacion(struct individuo *ind, float gmax[]) {
	int i;
	float v;

	if (TRUSSPROBL) {
		return;
	}

	ind->viol = 0;

	for (i = 0; i < NUMCONSTR; i++) {
		/* Inequiality constraints */
		if (i < NUMCONSTR - NUMEQCONSTR) {
			v = (ind->g[i] > 0)? ind->g[i]: 0;
		}
		/* Equality constraints */
		else {
			v = fabs(ind->g[i]);
		}

		if (v > gmax[i]) {
			gmax[i] = v;
		}
		ind->viol += v/gmax[i];
	}
}

/* Actualiza el espacio de creencias.
   La parte normativa se actualiza cada k generaciones */
void actualizaCreencias(struct individuo *pobl, struct creencias *esp, int t) {
	int aceptados[TOP + 3];
	int i, j, iinf, isup, dim, numHijo, sumando, celda, k = 20;
	float sup, inf;
	struct celda *nodoAct;

	/* Decide si realiza la actualizacin de la parte normativa */
	if (t%k == 0) {
		/* Elige a los aceptados para afectar a la parte normativa
		   del espacio de creencias */
		aceptar(pobl);
		for (i = 0; i < VARIABLES; i++) {
			iinf = 0;
			inf = pobl[0].variable[i];
			isup = 0;
			sup = pobl[0].variable[i];
			/* Ciclo para buscar los valores ms altos y bajos
			   de cada una de las variables */
			for (j = 1; j < TOP; j++) {
				if (pobl[j].variable[i] < inf) {
					iinf = j;
					inf = pobl[j].variable[i];
				}
				if (pobl[j].variable[i] > sup) {
					isup = j;
					sup = pobl[j].variable[i];
				}
			}

			/* Aplicacin de las reglas de actualizacin
			   de la parte normativa */
/*			if ( (inf < esp->l[i]) || (pobl[iinf].aptitud < esp->L[i] && pobl[iinf].factible) ) {
				if ( (sup > esp->u[i]) || (pobl[isup].aptitud < esp->U[i] && pobl[isup].factible) ) {
					if (inf < sup || inf < esp->l[i]) {
						esp->l[i] = inf;
						esp->L[i] = pobl[iinf].aptitud;
					}
					if (sup > inf || sup > esp->u[i]) {
						esp->u[i] = sup;
						esp->U[i] = pobl[isup].aptitud;
					}
				}
				else {
					if (inf < esp->u[i]) {
						esp->l[i] = inf;
						esp->L[i] = pobl[iinf].aptitud;
					}
				}
			}
			else if ( (sup > esp->u[i]) || (pobl[isup].aptitud < esp->U[i] && pobl[isup].factible) ) {
				if (sup > esp->l[i]) {
					esp->u[i] = sup;
					esp->U[i] = pobl[isup].aptitud;
				}
			}*/

			if ( (inf < esp->l[i]) || (pobl[iinf].aptitud < esp->L[i] && pobl[iinf].factible) ) {
				if (inf < esp->u[i]) {
					esp->l[i] = inf;
					esp->L[i] = pobl[iinf].aptitud;
				}
			}
			if ( (sup > esp->u[i]) || (pobl[isup].aptitud < esp->U[i] && pobl[isup].factible) ) {
				if (sup > esp->l[i]) {
					esp->u[i] = sup;
					esp->U[i] = pobl[isup].aptitud;
				}
			}
//printf("Dimensin %2d: %0.10f - %0.10f\n", i, esp->l[i], esp->u[i]);
		}
		/* Crea un nuevo rbol para los nuevos intervalos */
ftime(&tiempo1);
		expande(esp->raiz, pobl, esp);
ftime(&tiempo2);
diferencia += 1000*(tiempo2.time - tiempo1.time) + tiempo2.millitm - tiempo1.millitm;
	}

	/* Actualiza la parte de restricciones */

	/* Asocia a cada individuo con el nodo que lo contiene */
	ftime(&tiempo1);
	for (i = 0; i < TAMPOBL; i++) {
		numHijo = 0;
		/* Ciclo para verificar que el individuo se encuentre
		   dentro de los intervalos de la parte normativa */
		for (j = 0; j < VARIABLES; j++) {
			if (pobl[i].variable[j] < esp->l[j] ||
			    pobl[i].variable[j] > esp->u[j]) {
				numHijo = -1;
				break;
			}
		}
		if (numHijo == -1) {
			pobl[i].celda = NULL;
			continue;
		}

		for (nodoAct = esp->raiz; nodoAct->d[0] != -1; nodoAct = &(nodoAct->hijo[numHijo])) {
			numHijo = 0;
			sumando = 1;
			for (j = 0; j < TREEDIMS; j++) {
				dim = nodoAct->d[j];
				if (pobl[i].variable[dim] > (nodoAct->lnodo[dim] + nodoAct->unodo[dim])/2) {
					numHijo += sumando;
				}
				sumando += sumando;
			}
		}
		pobl[i].celda =  nodoAct;

		if (pobl[i].factible) {
			pobl[i].celda->factibles++;
			if (pobl[i].celda->factibles == 1) {
				if (pobl[i].celda->noFactibles == 0) {
					pobl[i].celda->clase = FACTIBLE;
				}
				else {
					pobl[i].celda->clase = SEMIFACTIBLE;
					if (pobl[i].celda->profundidad > 1) {
						expande(pobl[i].celda, pobl, esp);
						i = -1;
						continue;
					}
				}
			}
		}
		else {
			pobl[i].celda->noFactibles++;
			if (pobl[i].celda->noFactibles == 1) {
				if (pobl[i].celda->factibles == 0) {
					pobl[i].celda->clase = NO_FACTIBLE;
				}
				else {
					pobl[i].celda->clase = SEMIFACTIBLE;
					if (pobl[i].celda->profundidad > 1) {
						expande(pobl[i].celda, pobl, esp);
						i = -1;
						continue;
					}
				}
			}
		}
	}
	ftime(&tiempo2);
	diferencia += 1000*(tiempo2.time - tiempo1.time) + tiempo2.millitm - tiempo1.millitm;
}

void expande(struct celda *nodoAct, struct individuo *pobl, struct creencias *esp) {
	int i, j, numArbol, numHijo, sumando, dim, min;
	int sumMin[PRUEBAS_ARBOL][TREEDIMS + 1];
	float tmp;

	/* Crea y habilita a los hijos para formar el rbol */
	if (nodoAct->hijo == NULL) {
		nodoAct->hijo = (struct celda *) malloc(TREENODES*sizeof(struct celda));
		for (i = 0; i < TREENODES; i++) {
			nodoAct->hijo[i].padre = nodoAct;
			nodoAct->hijo[i].hijo = NULL;
			nodoAct->hijo[i].profundidad = nodoAct->profundidad - 1;
		}
	}
	for (i = 0; i < TREENODES; i++) {
		nodoAct->hijo[i].d[0] = -1;
	}

	/* Si es la raz, sus intervalos son los de la parte normativa */
	if (nodoAct->padre == NULL) {
		for (i = 0; i < VARIABLES; i++) {
			nodoAct->lnodo[i] = esp->l[i];
			nodoAct->unodo[i] = esp->u[i];
		}
	}

	if (VARIABLES > TREEDIMS) {
		for (numArbol = 0; numArbol < PRUEBAS_ARBOL; numArbol++) {
			/* Reiniciacin de los contadores de individuos factibles
			   de los hijos */
			for (i = 0; i < TREENODES; i++) {
				nodoAct->hijo[i].factibles = 0;
				nodoAct->hijo[i].noFactibles = 0;
			}

			/* Eleccin de las dimensiones por particionar al azar */
			sumMin[numArbol][1] = rnd(0, VARIABLES-1);
			sumMin[numArbol][2] = rnd(0, VARIABLES-2);
			sumMin[numArbol][3] = rnd(0, VARIABLES-3);
			if (sumMin[numArbol][2] >= sumMin[numArbol][1]) {
				sumMin[numArbol][2]++;
				if (sumMin[numArbol][3] >= sumMin[numArbol][1]) {
					sumMin[numArbol][3]++;
				}
				if (sumMin[numArbol][3] >= sumMin[numArbol][2]) {
					sumMin[numArbol][3]++;
				}
			}
			else {
				if (sumMin[numArbol][3] >= sumMin[numArbol][2]) {
					sumMin[numArbol][3]++;
				}
				if (sumMin[numArbol][3] >= sumMin[numArbol][1]) {
					sumMin[numArbol][3]++;
				}
			}
//printf("%d %d %d\n",sumMin[numArbol][1],sumMin[numArbol][2],sumMin[numArbol][3]);

			for (i = 0; i < TAMPOBL; i++) {
				numHijo = 0;
				for (j = 0; j < VARIABLES; j++) {
					if (pobl[i].variable[j] < nodoAct->lnodo[j] ||
					    pobl[i].variable[j] > nodoAct->unodo[j]) {
						numHijo = -1;
						break;
					}
				}
				if (numHijo == -1) {
					continue;
				}
				sumando = 1;
				for (j = 0; j < TREEDIMS; j++) {
					dim = sumMin[numArbol][j + 1];
					if (pobl[i].variable[dim] > (nodoAct->lnodo[dim] + nodoAct->unodo[dim])/2) {
						numHijo += sumando;
					}
					sumando += sumando;
				}
				if (pobl[i].factible) {
					nodoAct->hijo[numHijo].factibles++;
				}
				else {
					nodoAct->hijo[numHijo].noFactibles++;
				}
			}

			sumMin[numArbol][0] = 0;
			for (i = 0; i < TREENODES; i++) {
				sumMin[numArbol][0] += (nodoAct->hijo[i].factibles < nodoAct->hijo[i].noFactibles)? nodoAct->hijo[i].factibles: nodoAct->hijo[i].noFactibles;
			}
		}
		min = sumMin[0][0];
		numArbol = 0;
		for (i = 1; i < PRUEBAS_ARBOL; i++) {
			if (sumMin[i][0] < min) {
				min = sumMin[i][0];
				numArbol = i;
			}
		}
		
		for (i = 0; i < TREEDIMS; i++) {
			nodoAct->d[i] = sumMin[numArbol][i + 1];
		}
	}

	else {
		for (i = 0; i < VARIABLES; i++) {
			nodoAct->d[i] = i;
		}
	}

	for (i = 0; i < TAMPOBL; i++) {
		numHijo = 0;
		for (j = 0; j < VARIABLES; j++) {
			if (pobl[i].variable[j] < nodoAct->lnodo[j] ||
			    pobl[i].variable[j] > nodoAct->unodo[j]) {
				numHijo = -1;
				break;
			}
		}
		if (numHijo == -1) {
			continue;
		}
		sumando = 1;
		for (j = 0; j < TREEDIMS; j++) {
			dim = nodoAct->d[j];
			if (pobl[i].variable[dim] > (nodoAct->lnodo[dim] + nodoAct->unodo[dim])/2) {
				numHijo += sumando;
			}
			sumando += sumando;
		}
		if (pobl[i].factible) {
			nodoAct->hijo[numHijo].factibles++;
		}
		else {
			nodoAct->hijo[numHijo].noFactibles++;
		}
	}

	/* Asignacin del los intervalos a cada celda */
	for (i = 0; i < TREENODES; i++) {
		for (j = 0; j < VARIABLES; j++) {
			nodoAct->hijo[i].lnodo[j] = nodoAct->lnodo[j];
			nodoAct->hijo[i].unodo[j] = nodoAct->unodo[j];
		}
		numHijo = i;
		sumando = 1;
		for (j = 0; j < TREEDIMS; j++) {
			tmp = (nodoAct->lnodo[nodoAct->d[j]] + nodoAct->unodo[nodoAct->d[j]]) / 2;
			if (numHijo % (2*sumando)) {
				nodoAct->hijo[i].lnodo[nodoAct->d[j]] = tmp;
				numHijo -= sumando;
			}
			else {
				nodoAct->hijo[i].unodo[nodoAct->d[j]] = tmp;
			}
			sumando += sumando;
		}
	}

	/* Asignacin de la clase a la celda */
	for (i = 0; i < TREENODES; i++) {
		if (nodoAct->hijo[i].factibles > 0) {
			if (nodoAct->hijo[i].noFactibles > 0) {
				nodoAct->hijo[i].clase = SEMIFACTIBLE;
			}
			else {
				nodoAct->hijo[i].clase = FACTIBLE;
			}
		}
		else {
			if (nodoAct->hijo[i].noFactibles > 0) {
				nodoAct->hijo[i].clase = NO_FACTIBLE;
			}
			else {
				nodoAct->hijo[i].clase = DESCONOCIDA;
			}
		}
	}

	/* Llamada recursiva a expande */
	if (nodoAct->profundidad > 1) {
		for (i = 0; i < TREENODES; i++) {
			if (nodoAct->hijo[i].clase == SEMIFACTIBLE) {
				expande(&(nodoAct->hijo[i]), pobl, esp);
			}
		}
	}
}

void aceptar(struct individuo *pobl) {
	/* Ordenar descendentemente por aptitud, pero dejando factibles primero */
	qsort(pobl, TAMPOBL, sizeof(struct individuo), &compVictorias);
}

int compAptitud(const void *ind1, const void *ind2) {
	int dif;

	if ( ((struct individuo *)ind2)->factible && ((struct individuo *)ind1)->factible ) {
		if (((struct individuo *)ind2)->aptitud > ((struct individuo *)ind1)->aptitud) {
			return -1;
		}
		if (((struct individuo *)ind2)->aptitud < ((struct individuo *)ind1)->aptitud) {
			return 1;
		}
		return 0;
	}
		
	if ( ((struct individuo *)ind1)->factible ) {
		return -1;
	}
	if ( ((struct individuo *)ind2)->factible ) {
		return 1;
	}

	if (((struct individuo *)ind2)->viol > ((struct individuo *)ind1)->viol) {
		return -1;
	}
	if (((struct individuo *)ind2)->viol < ((struct individuo *)ind1)->viol) {
		return 1;
	}

	return 0;
}

void generarHijos(struct individuo *pobl, struct creencias *esp) {
	int i, j;
	float x;

	for (i = 0; i < TAMPOBL; i++) {
		for (j = 0; j < VARIABLES; j++) {
			pobl[i+TAMPOBL].variable[j] = pobl[i].variable[j];
			x = randomnormaldeviate();
			if (pobl[i].variable[j] < esp->l[j]) {
				pobl[i+TAMPOBL].variable[j] += fabs(x*(esp->u[j] - esp->l[j]));
			}
			else if (pobl[i].variable[j] > esp->u[j]) {
				pobl[i+TAMPOBL].variable[j] -= fabs(x*(esp->u[j] - esp->l[j]));
			}
			else if (pobl[i].celda == NULL) {
				pobl[i+TAMPOBL].variable[j] += x*(esp->u[j]-esp->l[j]);
			}
			else if (pobl[i].celda->clase == NO_FACTIBLE) {
				mueve(i, j, x, pobl, esp);
			}
			else {
				pobl[i+TAMPOBL].variable[j] += x*(pobl[i].celda->unodo[j] - pobl[i].celda->lnodo[j]);
			}
			if (pobl[i+TAMPOBL].variable[j] > esp->up[j]) {
				pobl[i+TAMPOBL].variable[j] = esp->up[j];
			}
			else if (pobl[i+TAMPOBL].variable[j] < esp->lp[j]) {
				pobl[i+TAMPOBL].variable[j] = esp->lp[j];
			}
		}
	}
}

void mueve(int indiv, int dim, float x, struct individuo *pobl, struct creencias *esp) {
	struct celda *celdaNueva;

	celdaNueva = cercana(SEMIFACTIBLE, pobl[indiv].celda);
	if (celdaNueva == NULL) {
		celdaNueva = cercana(DESCONOCIDA, pobl[indiv].celda);
	}

	if (celdaNueva == NULL) {
//		pobl[indiv+TAMPOBL].variable[dim] += x*(pobl[indiv].celda->unodo[dim] - pobl[indiv].celda->lnodo[dim]);
		pobl[indiv+TAMPOBL].variable[dim] += x*(esp->u[dim] - esp->l[dim]);
	}
	else {
/*		if (pobl[indiv+TAMPOBL].variable[dim] < celdaNueva->lnodo[dim]) {
			pobl[indiv+TAMPOBL].variable[dim] += fabs(x*(celdaNueva->lnodo[dim] - pobl[indiv+TAMPOBL].variable[dim]));
		}
		else if (pobl[indiv+TAMPOBL].variable[dim] > celdaNueva->unodo[dim]) {
			pobl[indiv+TAMPOBL].variable[dim] -= fabs(x*(pobl[indiv+TAMPOBL].variable[dim] - celdaNueva->unodo[dim]));
		}
		else {
			pobl[indiv+TAMPOBL].variable[dim] += x*(celdaNueva->unodo[dim] - celdaNueva->lnodo[dim]);
		}*/
		pobl[indiv+TAMPOBL].variable[dim] = x*(celdaNueva->unodo[dim] - celdaNueva->lnodo[dim]) + (celdaNueva->unodo[dim] + celdaNueva->lnodo[dim])/2;
	}
}

struct celda *cercana(int prioridad, struct celda *nodo) {
	int i;
	struct celda *res;

	for (; nodo->padre != NULL; nodo = nodo->padre) {
		for (i = 0; i < TREENODES; i++) {
			if (&(nodo->padre->hijo[i]) == nodo) {
				continue;
			}
			res = busca(prioridad, &(nodo->padre->hijo[i]));
			if (res != NULL) {
				return res;
			}
		}
	}
	return NULL;
}

struct celda *busca(int prioridad, struct celda *nodo) {
	int i;
	struct celda *res;

	if (nodo->d[0] == -1) {
		if (nodo->clase <= prioridad) {
			return nodo;
		}
		else {
			return NULL;
		}
	}

	for (i = 0; i < TREENODES; i++) {
		res = busca(prioridad, &(nodo->hijo[i]));
		if (res != NULL) {
			return res;
		}
	}
	return NULL;
}

/* Realiza el torneo de la poblacion de tamao 2*TAMPOBL.
   El nmero de enfrentamientos por individuo es c */
void selecciona(struct individuo *pobl) {
	int c = TAMPOBL/2;
	int i, j, contr, indmin;
	float min;
	char hayFactibles;

	/* Prueba para saber si hay factibles */
	for (i = 0; (!pobl[i].factible) || (i < 2*TAMPOBL); i++);
	hayFactibles = (i < 2*TAMPOBL)? 1: 0;

	/* Ciclo para realizar los torneos */
	for (i = 0; i < 2*TAMPOBL; i++) {
		pobl[i].victorias = 0;
		for (j = 0; j < c; j++) {
			contr = rnd(0, 2*TAMPOBL-2);
			contr = (contr>=i)? contr+1: contr;
			if ((pobl[i].factible && pobl[contr].factible)) {
				if (pobl[i].aptitud < pobl[contr].aptitud) {
					pobl[i].victorias++;
				}
			}
			else {
				if (!pobl[contr].factible) {
					if (pobl[i].factible) {
						pobl[i].victorias++;
					}
					else if (pobl[i].viol < pobl[contr].viol) {
						pobl[i].victorias++;
					}
				}
			}
/*			if (pobl[i].factible == pobl[contr].factible) {
				if (pobl[i].aptitud < pobl[contr].aptitud) {
					pobl[i].victorias++;
				}
			}
			else {
				if (pobl[i].factible) {
					pobl[i].victorias++;
				}
			}*/
		}
	}

	/* Lleva a cabo el elitismo, asignando ms vicorias al mejor individuo */
	for (i = 0; !pobl[i].factible && i < 2*TAMPOBL; i++);
	if (i < 2*TAMPOBL) {
		min = pobl[i].aptitud;
		indmin = i;
		for (; i < 2*TAMPOBL; i++) {
			if (pobl[i].factible && pobl[i].aptitud < min) {
				indmin = i;
				min = pobl[i].aptitud;
			}
		}
		pobl[indmin].victorias = c + 1;
	}

	/* Ordenar descendentemente por nmero de victorias */
	qsort(pobl, 2*TAMPOBL, sizeof(struct individuo), &compVictorias);
//	nuevoInd(&(pobl[TAMPOBL - 1]));
}

int compVictorias(const void *ind1, const void *ind2) {
	int dif;
	dif = ((struct individuo *)ind2)->victorias - ((struct individuo *)ind1)->victorias;
	return dif;
}

void nuevoInd(struct individuo *ind) {
	int j;
	float l[VARIABLES], u[VARIABLES];

	limites(l, u);
	for (j = 0; j < VARIABLES; j++) {
		ind->variable[j] = rndreal(l[j], u[j]);
	}
}

/* Funcin para encontrar las aptitudes mnima y mxima de pobl */
void extremos(int *indmin, int *indmax, struct individuo *pobl) {
	int i, sumados = 0;
	float min, max, med = 0.0;

	/* Se prefieren soluciones factibles */
	for (i = 0; !pobl[i].factible && i < TAMPOBL; i++);
	if (i < TAMPOBL) {
		max = min = pobl[i].aptitud;
		*indmin = *indmax = i;
		for (; i < TAMPOBL; i++) {
			if (pobl[i].factible) {
				if (pobl[i].aptitud < min) {
					*indmin = i;
					min = pobl[i].aptitud;
				}
				if (pobl[i].aptitud > max) {
					*indmax = i;
					max = pobl[i].aptitud;
				}
				med += pobl[i].aptitud;
				sumados++;
			}
		}
	}
	else {
		max = min = pobl[0].viol;
		*indmin = *indmax = 0;
		for (i = 0; i < TAMPOBL; i++) {
			if (pobl[i].viol < min) {
				*indmin = i;
				min = pobl[i].viol;
			}
			if (pobl[i].viol > max) {
				*indmax = i;
				max = pobl[i].viol;
			}
			med += pobl[i].aptitud;
			sumados++;
		}
	}
	media = med/sumados;
}
